var gulp = require('gulp');
var ts = require('gulp-typescript');
var tsProject = ts.createProject('tsconfig.json');
var runSequence = require('run-sequence');
var sourcemaps = require('gulp-sourcemaps');
var Server = require('karma').Server;
var remapIstanbul = require('remap-istanbul/lib/gulpRemapIstanbul');
var path = require('path');

gulp.task('compile', function() {
  var tsResult = gulp.src(['./**/*.ts', '!node_modules/**/*.ts', '!typings/**/*.ts', '!compiled/**/*.ts'])
      .pipe(sourcemaps.init())
      .pipe(ts(tsProject));

  return tsResult.js.pipe(sourcemaps.write({
        sourceRoot: function (file) {
          var sourceFile = path.join(file.cwd, file.sourceMap.file);
          return path.relative(path.dirname(sourceFile), file.cwd);
        }
      })).pipe(gulp.dest('compiled'));
});

gulp.task('watch', ['compile'], function() {
  gulp.watch('./src/**/*.ts', ['compile']);
});

gulp.task('unit-test', ['compile'], function (done) {
  new Server({
    configFile: __dirname + '/karma.conf.js',
    singleRun: true
  }, function(exitCode) {
    console.log('Test Done with exit code: ' + exitCode);
    remapCoverage();
    if(exitCode === 0) {
      done();
    } else {
      done('Unit test failed.');
    }
  }).start();

});

function remapCoverage() {
  console.log('Remapping coverage to TypeScript format...');
  return gulp.src('/coverage/coverage-final.json')
      .pipe(remapIstanbul({
        reports: {
          'lcovonly': './coverage/remap/lcov.info',
          'json': './coverage/remap/coverage.json',
          'html': './coverage/remap/html-report',
          'text-summary': './coverage/remap/text-summary.txt'
        }
      }))
      .on('finish', function () {
        console.log('Remapping done! View the result in coverage/remap/html-report');
      });
}